"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.activate = void 0;
const vscode = require("vscode");
const formatter = require("./formatter");
const autocomplete = require("./autocomplete");
const escaper = require("./escaper");
const massformatter = require("./massformatter");
const normalizer = require("./normalizer");
const outliner = require("./outliner");
let toJsonButton;
let toRcjsonButton;
let massFormatCancelButton;
let ac;
let f;
let esc;
let a;
let mf;
let n;
let o;
let formatterWorkingState = false;
let outputChannel;
let jsonFileLanguageRegister = {};
function activate(context) {
    outputChannel = vscode.window.createOutputChannel("ROOMLE formatter");
    f = new formatter.Formatter();
    f.outputChannel = outputChannel;
    n = new normalizer.Normalizer();
    n.outputChannel = outputChannel;
    a = new autocomplete.Autocomplete();
    esc = new escaper.Escaper();
    mf = new massformatter.MassFormatter();
    mf.outputChannel = outputChannel;
    o = new outliner.RcjsonOutliner();
    o.outputChannel = outputChannel;
    console.log('RML Formatter activating');
    toJsonButton = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Right, 200);
    toJsonButton.command = 'roomletool.jsonMultilineEscape';
    toJsonButton.text = '>JSON';
    massFormatCancelButton = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Right, 1);
    massFormatCancelButton.command = 'roomletool.cancelFormatting';
    massFormatCancelButton.text = 'STOP MASS FORMATTING';
    mf.massFormatCancelButton = massFormatCancelButton;
    toRcjsonButton = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Right, 199);
    toRcjsonButton.command = 'roomletool.jsonMultilineUnEscape';
    toRcjsonButton.text = '>RC-JSON';
    context.subscriptions.push(vscode.languages.registerDocumentSymbolProvider({ scheme: "*", language: "rcjson" }, o));
    outputChannel.appendLine("Roomle Formatter activated.");
    setWorkingState(false);
    let autoCompleteProvider = vscode.languages.registerCompletionItemProvider('rcjson', {
        provideCompletionItems(document, position, token, context) {
            return null;
        }
    });
    ac = autoCompleteProvider;
    context.subscriptions.push(autoCompleteProvider);
    context.subscriptions.push(toJsonButton);
    context.subscriptions.push(toRcjsonButton);
    vscode.window.onDidChangeActiveTextEditor((textEditor) => {
        if (!textEditor) {
            return;
        }
        if (textEditor && (textEditor.document.languageId === 'rcjson') && !formatterWorkingState) {
            runFormatterAndAutocomplete(true, false, false, true);
        }
    });
    vscode.workspace.onDidOpenTextDocument((textDocument) => {
        if (formatterWorkingState) {
            return;
        }
        if (decideIfJsonIsRcJson(textDocument)) {
            vscode.languages.setTextDocumentLanguage(textDocument, "rcjson");
            return;
            // will fire another onDidOpenTextDocument right away
        }
        if (!formatterWorkingState && textDocument && textDocument.languageId === 'rcjson') {
            runFormatterAndAutocomplete(true, false, false, true);
        }
        showHideToRcJsonButtons();
    });
    function decideIfJsonIsRcJson(textDocument) {
        var path = textDocument.fileName;
        var explicitJson = jsonFileLanguageRegister[path] === "ExplicitJSON";
        var explicitRcJson = jsonFileLanguageRegister[path] === "ExplicitRCJSON";
        var pathContainsComponents = textDocument.fileName.indexOf("component") >= 0;
        if (textDocument.languageId === 'json') {
            var text = textDocument.getText();
            var matches = text.match(/\"id\":/);
            if (explicitJson) {
                return false;
            }
            else if ((matches && matches.length > 0 && pathContainsComponents && !explicitJson) || explicitRcJson) {
                return true;
            }
        }
        return false;
    }
    let disposableFormatAndFeedAutocomplete = vscode.commands.registerCommand('roomletool.formatRoomleComponent', () => {
        if (!formatterWorkingState) {
            runFormatterAndAutocomplete(true, true, false, true);
        }
    });
    let disposableFormatNormalize = vscode.commands.registerCommand('roomletool.formatNormalizeRoomleComponent', () => {
        if (!formatterWorkingState) {
            runFormatterAndAutocomplete(true, true, true, true);
        }
    });
    let disposableFormat = vscode.commands.registerCommand('roomletool.formatNoAutocomplete', () => {
        if (!formatterWorkingState) {
            runFormatterAndAutocomplete(false, true, false, true);
        }
    });
    let disposableFormatAllInWorkspace = vscode.commands.registerCommand('roomletool.formatAllJsonsInCurrentWorkspace', () => {
        if (!formatterWorkingState) {
            formatAllTabsInWorkspace();
        }
    });
    let disposableFormatAll = vscode.commands.registerCommand('roomletool.formatAllOpenRoomleJsons', () => {
        if (!formatterWorkingState) {
            formatAllTabs();
        }
    });
    let disposableFormatNormalizeOverride = vscode.commands.registerCommand('roomletool.overrideFormatNormalizeWhileAllTabs', () => {
        runFormatterAndAutocomplete(false, true, true, false);
    });
    let disposableFormatOverride = vscode.commands.registerCommand('roomletool.overrideFormatWhileAllTabs', () => {
        runFormatterAndAutocomplete(false, true, false, false);
    });
    let disposableCancelFormatting = vscode.commands.registerCommand('roomletool.cancelFormatting', () => {
        f.shouldCancel = true;
        mf.shouldCancel = true;
    });
    let disposableEscape = vscode.commands.registerCommand('roomletool.jsonMultilineEscape', () => {
        if (vscode.window.activeTextEditor && !formatterWorkingState) {
            var escaped = esc.getEscaped(getDocument());
            var textDocument = vscode.window.activeTextEditor.document;
            jsonFileLanguageRegister[textDocument.fileName] = "ExplicitJSON";
            apply(escaped);
            vscode.languages.setTextDocumentLanguage(textDocument, "json");
            vscode.window.showInformationMessage("Escaped, handling as JSON");
        }
    });
    let disposableUnEscape = vscode.commands.registerCommand('roomletool.jsonMultilineUnEscape', () => {
        if (vscode.window.activeTextEditor && !formatterWorkingState) {
            var unescaped = esc.getUnescaped(getDocument());
            var textDocument = vscode.window.activeTextEditor.document;
            jsonFileLanguageRegister[textDocument.fileName] = "ExplicitRCJSON";
            apply(unescaped);
            vscode.languages.setTextDocumentLanguage(textDocument, "rcjson");
            vscode.window.showInformationMessage("Unescaped, handling as RC-JSON");
        }
    });
    function runFormatterAndAutocomplete(feedAutocomplete, applyFormatting, normalize, showVscodeWindow) {
        if (!feedAutocomplete && !applyFormatting) {
            if (showVscodeWindow) {
                vscode.window.showWarningMessage("Request not to format neither feed autocomplete. This should not happen");
            }
        }
        if (vscode.window.activeTextEditor) {
            var escaped = esc.getEscaped(getDocument());
            var clean = escaped.replace(/\r?\n|\r/g, '');
            var obj;
            var parseFailed = false;
            try {
                obj = JSON.parse(clean);
            }
            catch (_a) {
                parseFailed = true;
            }
            if (obj === null || parseFailed) {
                if (showVscodeWindow) {
                    vscode.window.showErrorMessage("JSON does not parse: " + vscode.window.activeTextEditor.document.fileName);
                }
                outputChannel.appendLine("[ERROR] " + vscode.window.activeTextEditor.document.fileName + " JSON does not parse.");
                return;
            }
            if (obj.id === undefined) {
                if (showVscodeWindow) {
                    vscode.window.showErrorMessage("JSON does not contain ID: " + vscode.window.activeTextEditor.document.fileName);
                }
                outputChannel.appendLine("[INFO] " + vscode.window.activeTextEditor.document.fileName + " JSON does not contain ID, formatting skipped.");
                return;
            }
            if (normalize) {
                try {
                    obj = n.normalizeComponent(obj);
                }
                catch (_b) {
                    outputChannel.appendLine("[WARNING] FAILED TO NORMALIZE " + obj.id);
                    if (showVscodeWindow) {
                        vscode.window.showErrorMessage("FAILED TO NORMALIZE");
                    }
                }
            }
            obj = f.formatRoomleJson(obj, showVscodeWindow);
            if (f.shouldCancel) {
                f.shouldCancel = false;
                if (showVscodeWindow) {
                    vscode.window.showWarningMessage("Job's cancelled");
                }
            }
            else {
                var formated = JSON.stringify(obj, null, 4);
                var unescaped = esc.getUnescaped(formated);
                if (feedAutocomplete) {
                    let identifiers = a.findAutocompleteEntries(obj, formated);
                    let autoCompleteProvider = vscode.languages.registerCompletionItemProvider('rcjson', {
                        provideCompletionItems(document, position, token, context) {
                            let simpleCompletions = [];
                            identifiers.forEach(id => {
                                let entryLabel = id.label;
                                let simpleCompletionKind = vscode.CompletionItemKind.Text;
                                switch (id.kind) {
                                    case autocomplete.AutocompleteEntryKind.na:
                                        break;
                                    case autocomplete.AutocompleteEntryKind.InternalVariable:
                                        simpleCompletionKind = vscode.CompletionItemKind.Variable;
                                        break;
                                    case autocomplete.AutocompleteEntryKind.Parameter:
                                        simpleCompletionKind = vscode.CompletionItemKind.Enum;
                                        entryLabel = "par " + entryLabel;
                                        break;
                                    case autocomplete.AutocompleteEntryKind.SubComponentInternalId:
                                        break;
                                    case autocomplete.AutocompleteEntryKind.Superseding:
                                        simpleCompletionKind = vscode.CompletionItemKind.Reference;
                                        break;
                                    case autocomplete.AutocompleteEntryKind.Value:
                                        simpleCompletionKind = vscode.CompletionItemKind.EnumMember;
                                        entryLabel = "val " + entryLabel;
                                        break;
                                    case autocomplete.AutocompleteEntryKind.Keyword:
                                        simpleCompletionKind = vscode.CompletionItemKind.Keyword;
                                        break;
                                }
                                const simpleCompletion = new vscode.CompletionItem(entryLabel);
                                simpleCompletion.insertText = id.output;
                                simpleCompletion.documentation = id.documentation;
                                simpleCompletion.kind = simpleCompletionKind;
                                simpleCompletions.push(simpleCompletion);
                            });
                            return simpleCompletions;
                        }
                    });
                    ac.dispose();
                    ac = autoCompleteProvider;
                    context.subscriptions.push(autoCompleteProvider);
                }
                if (applyFormatting) {
                    apply(unescaped);
                }
                if (showVscodeWindow) {
                    if (applyFormatting && normalize)
                        vscode.window.showInformationMessage("Formatter: formatted & normalized");
                    else if (applyFormatting) {
                        vscode.window.showInformationMessage("Formatter: formatted");
                    }
                    else if (normalize) {
                        vscode.window.showInformationMessage("Formatter: normalized");
                    }
                    else {
                        vscode.window.showInformationMessage("Formatter: analysed");
                    }
                }
            }
        }
    }
    function showHideToRcJsonButtons() {
        var _a;
        var textDocument = (_a = vscode.window.activeTextEditor) === null || _a === void 0 ? void 0 : _a.document;
        if (textDocument && textDocument.languageId === 'rcjson') {
            toRcjsonButton.hide();
            toJsonButton.show();
        }
        else if (textDocument && textDocument.languageId === 'json') {
            toJsonButton.hide();
            toRcjsonButton.show();
        }
        else {
            toJsonButton.hide();
            toRcjsonButton.hide();
        }
    }
    function setWorkingState(working) {
        if (working) {
            massFormatCancelButton.show();
            toJsonButton.hide();
            toRcjsonButton.hide();
        }
        else {
            massFormatCancelButton.hide();
            showHideToRcJsonButtons();
            f.shouldCancel = true;
            mf.shouldCancel = true;
        }
        formatterWorkingState = working;
    }
    function getDocument() {
        if (vscode.window.activeTextEditor) {
            var json = vscode.window.activeTextEditor.document.getText();
            return json;
        }
        else {
            return "";
        }
    }
    function apply(data) {
        if (vscode.window.activeTextEditor) {
            vscode.window.activeTextEditor.edit(editBuilder => {
                if (!vscode.window.activeTextEditor) {
                    return;
                }
                editBuilder.replace(new vscode.Range(0, 0, vscode.window.activeTextEditor.document.lineCount, 0), data);
            });
        }
    }
    function formatAllTabs() {
        return __awaiter(this, void 0, void 0, function* () {
            setWorkingState(true);
            yield mf.formatAllFiles();
            setWorkingState(false);
        });
    }
    function formatAllTabsInWorkspace() {
        return __awaiter(this, void 0, void 0, function* () {
            let confirmResult = yield vscode.window.showInputBox({
                prompt: 'This will open, format and save all JSONs in your workspace. You can also mass normalize with this feature.',
                placeHolder: 'Type "yes" or "normalize" to confirm that you know what you are going to do.'
            });
            var alsoNormalize = false;
            if (confirmResult === "yes")
                alsoNormalize = false;
            else if (confirmResult === "normalize")
                alsoNormalize = true;
            else
                return;
            setWorkingState(true);
            yield mf.formatAllFilesInWorkspace(alsoNormalize);
            setWorkingState(false);
        });
    }
}
exports.activate = activate;
// this method is called when your extension is deactivated
function deactivate() { }
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map