"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MassFormatter = void 0;
const vscode = require("vscode");
class MassFormatter {
    constructor() {
        this.shouldCancel = false;
    }
    formatAllFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            let u = 0;
            this.shouldCancel = false;
            let visited = [];
            while (true) {
                let editor = vscode.window.activeTextEditor;
                if (editor) {
                    let filename = editor.document.fileName;
                    if (filename.endsWith("json")) {
                        yield vscode.commands.executeCommand("roomletool.overrideFormatWhileAllTabs");
                    }
                    if (this.shouldCancel || visited.indexOf(filename) > -1) {
                        break;
                    }
                    else {
                        visited.push(filename);
                    }
                    u++;
                }
                else {
                    break;
                }
                yield vscode.commands.executeCommand("workbench.action.nextEditor");
            }
            vscode.window.showInformationMessage("Formatted " + u + " files.");
            if (this.outputChannel !== undefined) {
                this.outputChannel.appendLine("Formatted " + u + " files.");
            }
        });
    }
    formatAllFilesInWorkspace(normalize) {
        return __awaiter(this, void 0, void 0, function* () {
            let u = 0;
            this.shouldCancel = false;
            const files = yield vscode.workspace.findFiles('**/*.json');
            const filesLength = files.length;
            let lastCountDone = 0;
            files.sort();
            if (this.massFormatCancelButton) {
                this.massFormatCancelButton.text = "MASS FORMATTING 0 / " + filesLength + " STOP";
            }
            for (var i = 0; i < filesLength; i++) {
                yield vscode.workspace.openTextDocument(files[i]).then((doc) => __awaiter(this, void 0, void 0, function* () {
                    yield vscode.window.showTextDocument(doc);
                }));
                if (normalize) {
                    yield vscode.commands.executeCommand("roomletool.overrideFormatNormalizeWhileAllTabs");
                }
                else {
                    yield vscode.commands.executeCommand("roomletool.overrideFormatWhileAllTabs");
                }
                const activeEditor = vscode.window.activeTextEditor;
                if (activeEditor)
                    yield activeEditor.document.save();
                yield vscode.commands.executeCommand('workbench.action.closeActiveEditor');
                u++;
                if (u >= lastCountDone + 10) {
                    if (this.massFormatCancelButton) {
                        this.massFormatCancelButton.text = "MASS FORMATTING " + u + " / " + filesLength + " STOP";
                        lastCountDone = u;
                    }
                }
                if (this.shouldCancel) {
                    break;
                }
            }
            if (this.outputChannel !== undefined) {
                this.outputChannel.appendLine("Formatted " + u + " files.");
            }
            vscode.window.showInformationMessage("Format all JSONs in workspace finished.");
        });
    }
}
exports.MassFormatter = MassFormatter;
//# sourceMappingURL=massformatter.js.map